<?php
/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was written by:
 * @author Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 * 
 * 
 * 
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */


/**
 * backward compatibility: the DIR_SEP constant isn't used anymore
 */
if (! defined('DIR_SEP')) {
	define('DIR_SEP', DIRECTORY_SEPARATOR);
}
/**
 * backward compatibility: the PATH_SEPARATOR constant is availble since 4.3.0RC2
 */
if (! defined('PATH_SEPARATOR')) {
	define('PATH_SEPARATOR', OS_WINDOWS ? ';' : ':');
}


/**
 * Set the KREXX_DIR constant up with the absolute path to kreXX files.
 * If it is
 * not defined, include_path will be used. Set KREXX_DIR only if any other module
 * or application has not already set it up.
 */
if (! defined('KREXX_DIR')) {
	define('KREXX_DIR', dirname(__FILE__) . DIRECTORY_SEPARATOR);
}

/**
 * This constant sets the maximum strings of strings that will be shown
 * as they are.
 * Longer strings will be truncated with this length, and
 * their `full form` will be shown in a child node.
 */
if (! defined('KREXX_TRUNCATE_LENGTH')) {
	define('KREXX_TRUNCATE_LENGTH', 50);
}

/**
 * Register an alias function for object analysis, so you will not have
 * to type \kreXX::open($data); all the time
 * 
 * 
 * @param mixed $data
 */
function krexx() {
	$args = func_get_args();
	return call_user_func_array(
			array('kreXX', 'open'), $args
	);
	// \kreXX::open($data);
}

include_once KREXX_DIR . 'kreXXRender.php';
include_once KREXX_DIR . 'kreXXInternals.php';

/**
 * kreXX interface
 *
 * This class hosts the public functions which allow access to the kreXX debug features
 *
 * @package kreXX
 */
class kreXX extends \kreSpace\kreXXInternals {
	
	/**
	 * Here we save all timekeeping stuff
	 * 
	 * @var string array
	 */
	protected static $timekeeping = array(); 
	
	/**
	 * Remembers, if the header or footer must be printed
	 *
	 * @var integer
	 */
	protected static $startRun = 0;
	
	
	protected static $counterCache = array();
	

	/**
	 * Resets the timer and takes a "moment"
	 * @param string $string
	 */
	public Static function timerStart() {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// reset what we had before.
		self::$timekeeping = array();
		self::$counterCache = array();
		self::$timekeeping[] = $_SERVER['REQUEST_URI'];
		
		
		self::timerMoment('start');
	}
	
	/**
	 * Takes a "moment"
	 * @param string $string
	 */
	public static function timerMoment($string) {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// static $counterCache = array();

		// did we use this one before?
		if (isset(self::$counterCache[$string])) {
			// add another to the counter
			self::$counterCache[$string]++;
			self::$timekeeping['['.self::$counterCache[$string].']' . $string] =  microtime(true);
		} else {
			// first time counter, set it to 1
			self::$counterCache[$string] = 1;
			self::$timekeeping[$string] =  microtime(true);
		}
	}
	
	/**
	 * Takes a "moment" and outputs the timer in a analyse file
	 * @param unknown $string
	 */
	public static function timerEnd() {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		self::timerMoment('end');
		// . . . and we are done
		self::dump(self::miniBenchTo(self::$timekeeping), 'kreXX timer');

	}

	/**
	 * Creates a Reflection Class of the $data and dumps it's content
	 * 
	 * @param unknown $data
	 * @access public
	 */
// 	public  static function dumpReflection($data) {
// 		// disabled ?
// 		//
// 		if (! self::isEnabled()) {
// 			return false;
// 		}
// 		if (is_object($data)) {
// 			// get the reflection
// 			$ref = new ReflectionClass($data);
// 			// get all methods
// 			$functions = get_class_methods($ref);
// 			// define blacklist, these need some arguments
// 			$blacklist = array();
// 			$blacklist[] = '__construct';					// Constructs a ReflectionClass
// 			$blacklist[] = 'export';						// Exports a class
// 			$blacklist[] = 'getConstant';					// Gets defined constant
// 			$blacklist[] = 'getMethod';						// Gets a ReflectionMethod for a class method
// 			$blacklist[] = 'getProperty';					// Gets a ReflectionProperty for a class's property
// 			$blacklist[] = 'getStaticPropertyValue';		// Gets static property value
// 			$blacklist[] = 'hasConstant';					// Checks if constant is defined
// 			$blacklist[] = 'hasMethod';						// Checks if method is defined
// 			$blacklist[] = 'hasProperty';					// Checks if property is defined
// 			$blacklist[] = 'implementsInterface';			// Implements interface
// 			$blacklist[] = 'isInstance';					// Checks class for instance
// 			$blacklist[] = 'isSubclassOf';					// Checks if a subclass
// 			$blacklist[] = 'newInstance';					// Creates a new class instance from given arguments.
// 			$blacklist[] = 'newInstanceArgs';				// Creates a new class instance from given arguments.
// 			$blacklist[] = 'newInstanceWithoutConstructor';	// Creates a new class instance without invoking the constructor.
// 			$blacklist[] = 'setStaticPropertyValue';		// Sets static property value
			
// 			// and parse them all
// 			foreach ($functions as $function) {
// 				if (!in_array($function, $blacklist)){
// 					// but only when not blacklisted
// 					$output[$function] = $ref->$function();
// 				}
// 			}
// 			$output['hasMethod(getFieldnames)'] = $ref->hasMethod('getFieldnames');
// 		} else {
// 			$output = 'Not an object.';
// 		}
// 		self::dump($output, 'Data from the class reflection');
// 	}
	
	/**
	 * Dumps the SQL Log
	 * 
	 * @param \string $uniqueId
	 *        	ID of the Log, leave blank for everything (which micht be a little too much, uncached ~350 entries)
	 * @param \integer $count
	 *        	Anzahl der auzugerbenen Queries. Bei Drupal können es mal locker über 600 Stück sein, was eher sinnlos ist, und eventuell den PHP Speicher sprengt.
	 *        	
	 */
	public static function dumpSql($uniqueId = 'kreXX', $count = 0) {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render Typo3, if possible
		//
		// if (isset($GLOBALS['TYPO3_DB']->debug_lastBuiltQuery)) {
		// kreXX::dump($GLOBALS['TYPO3_DB']->debug_lastBuiltQuery, 'Last SQL Query from Typo3');
		// }
		
		// render Drupal, if possible
		//
		if (class_exists('Database')) {
			$data = Database::getLog($uniqueId);
			if ((int) $count == 0) {
				// Voreinstellungen aus dem Framework nehmen
				$countQueries = variable_get('include_kreXX_count_queries', 0);
				$data = array_slice($data, $countQueries * - 1, $countQueries, true);
			}
			
			self::dump($data, 'Last SQL Query from Drupal');
		}
	}
	
	/**
	 * Prints all methods and magic functions in this class
	 * Wrapper for the Dump method
	 *
	 * @param mixed $data,...        	
	 * @access public
	 */
	Public Static function open($data) {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		self::dump($data, 'Standard dump');
	}
	
	/**
	 * Starts the Drupal SQL Logging.
	 * The $uniqueId can be used to narrow down the dumpSql function
	 *
	 * @param \string $uniqueId        	
	 */
	public static function startDrupalLogging($uniqueId) {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		if (class_exists('Database') && method_exists(Database, 'startLog')) {
			Database::startLog($uniqueId);
		} else {
			// sure this is a Drupal?
			self::dump('Class "Database" not found');
		}
	}
	
	/**
	 * Prints out all currently defined global variables
	 */
	public static function variables() {
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump($GLOBALS, 'Global variables');
	}
	
	/**
	 * Prints a debug backtrace
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function backtrace() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(debug_backtrace(), 'Backtrace');
	}
	
	/**
	 * Prints a list of all currently declared classes.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function classes() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_declared_classes(), 'Declared classes');
	}
	
	/**
	 * Prints a list of all currently declared interfaces (PHP5 only).
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function interfaces() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_declared_interfaces(), 'Declared unterfaces');
	}
	
	/**
	 * Prints a list of all currently included (or required) files.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function includes() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_included_files(), 'Included (or required) files');
	}
	
	/**
	 * Prints a list of all currently declared functions.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function functions() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_defined_functions(), 'Declared functions');
	}
	
	/**
	 * Prints a list of all currently declared constants.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function defines() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_defined_constants(), 'declared constants (defines)');
	}
	
	/**
	 * Prints a list of all currently loaded PHP extensions.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function extensions() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(get_loaded_extensions(), 'loaded PHP extensions');
	}
	
	/**
	 * Prints a list of all HTTP request headers.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function headers() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(getallheaders(), 'HTTP request headers');
	}
	
	/**
	 * Prints a list of the configuration settings read from <i>php.ini</i>
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function phpini() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		$what = 'Configuration settings read from <code><b>' . get_cfg_var('cfg_file_path') . '</b></code>';
		if (get_cfg_var('cfg_file_path') === false) {
			self::dump('Phpini is not readable.', $what);
			return;
		}
		
		// render it
		//
		self::dump(parse_ini_file(get_cfg_var('cfg_file_path'), true), $what);
	}
	
	/**
	 * Prints a list of all your configuration settings.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function conf() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		self::dump(ini_get_all(), 'Configuration settings');
	}
	
	/**
	 * Prints a list of the specified directories under your <i>include_path</i> option.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function path() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Specified directories under your <code><b>include_path</b></code> option.';
		self::dump(explode(PATH_SEPARATOR, ini_get('include_path')), $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_REQUEST</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function request() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>$_REQUEST</b></code> array.';
		self::dump($_REQUEST, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_GET</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function get() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		// render it
		//
		$what = 'Values from the <code><b>$_GET</b></code> array';
		self::dump($_GET, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_POST</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function post() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>$_POST</b></code> array';
		self::dump($_POST, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_SERVER</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function server() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>$_SERVER</b></code> array';
		self::dump($_SERVER, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_COOKIE</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function cookie() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>$_COOKIE</b></code> array';
		self::dump($_COOKIE, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_ENV</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function env() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'values from the <code><b>$_ENV</b></code> array';
		self::dump($_ENV, $what);
	}
	
	/**
	 * Prints a list of all the values from the <i>$_SESSION</i> array.
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function session() {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>$_SESSION</b></code> array';
		self::dump($_SESSION, $what);
	}
	
	/**
	 * Prints a list of all the values from an INI file.
	 *
	 * @param string $ini_file        	
	 *
	 * @access public
	 * @static
	 *
	 */
	Public Static Function ini($ini_file) {
		
		// disabled ?
		//
		if (! self::isEnabled()) {
			return;
		}
		
		// read it
		//
		if (! $_ = @parse_ini_file($ini_file, 1)) {
			self::dump('File "' . $ini_file . '" is not readable.', $what);
			return;
		}
		
		// render it
		//
		$what = 'Values from the <code><b>' . realpath($ini_file) ? realpath($ini_file) : $ini_file . '</b></code> INI file.';
		self::dump($_, $what);
	}
	
	
	/**
	 * Enable kreXX
	 *
	 * @return boolean
	 * @access public
	 * @static
	 *
	 */
	Public Static Function enable() {
		return true === self::isEnabled(true);
	}
	
	/**
	 * Disable kreXX
	 *
	 * @return boolean
	 * @access public
	 * @static
	 *
	 */
	Public Static Function disable() {
		return false === self::isEnabled(false);
	}
}
