<?php

/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was written by:
 * @author Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 *
 *
 *
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */
namespace kreSpace;

/**
 * kreXX internals
 *
 * This class hosts the internal analysis functions
 *
 * @package kreXX
 */
class kreXXInternals extends kreXXRender {
	
	/**
	 * Fallback settings, in case there is nothing in the config ini
	 *
	 * @var array
	 */
	protected static $configFallback = array (
			'skin' => array (
					'selected' => 'schablon.com' 
			),
			'header' => array (
					'doctype' => '<!DOCTYPE html>' 
			),
			'logging' => array (
					'folder' => 'log',
					'maxfiles' => '10' 
			),
			'output' => array (
					'destination' => 'frontend',
					'useCookies' => 'false',
					'maxCall' => '10',
					'disabled' => 'false' 
			),
			'deep' => array (
					'analyseProtected' => 'false',
					'analysePrivate' => 'false',
					'analyseMethods' => 'true',
					'analyseTraversable' => 'true',
					'debugMethods' => 'debug,toArray,__toString,toString',
					'level' => '5' 
			) 
	);
	
	/**
	 * Here we save all arrays, as well as all object
	 * hashes, to prevent recussions
	 * [0] -> array with markers
	 * [1] -> object hashes
	 *
	 * @var array
	 */
	protected static $recussionHive = array ();
	
	/**
	 *
	 * @param unknown $arg_t        	
	 * @param string $arg_ra        	
	 * @return array
	 * @see http://php.net/manual/de/function.microtime.php
	 * @author gomodo at free dot fr
	 */
	protected static function miniBenchTo($arg_t) {
		$tttime = round((end($arg_t) - $arg_t['start']) * 1000, 4);
		$ar_aff['url'] = $_SERVER['REQUEST_URI'];
		$ar_aff['total_time'] = $tttime;
		$prv_cle = 'start';
		$prv_val = $arg_t['start'];
		
		foreach ($arg_t as $cle => $val) {
			if ($cle != 'start') {
				// calculate thie time
				$prcnt_t = round(((round(($val - $prv_val) * 1000, 4) / $tttime) * 100), 1);
				$ar_aff[$prv_cle . ' -> ' . $cle] = $prcnt_t . '%';
				$prv_val = $val;
				$prv_cle = $cle;
			}
		}
		return $ar_aff;
	}
	
	/**
	 * Get\Set kreXX state: whether it is enabled or disabled
	 *
	 * @param boolean $state        	
	 * @return boolean
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function isEnabled($state = null) {
		static $_ = true;
		
		// set
		//
		if (isset($state)) {
			$_ = (boolean) $state;
		}
		
		// disabled globally in the ini?
		if (self::_config('output', 'disabled') == 'true') {
			return false;
		}
		
		// cookies?
		if (self::_config('output', 'useCookies') == 'true') {
			if ($_COOKIE['kreXXDebug'] == 'yes') {
				$debugSession = true;
			} else {
				$debugSession = false;
			}
			
			if ($debugSession === true && $_ === true) {
				return true;
			} else {
				return false;
			}
		}
		// not using cookies
		return $_;
	}
	
	/**
	 * Returns values from kreXX's configuration
	 *
	 * @param string $group        	
	 * @param string $name        	
	 * @param mixed $fallback        	
	 * @return mixed
	 *
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _config($group, $name) {
		static $_config = array ();
		
		// not loaded ?
		//
		if (empty($_config)) {
			$_config = (array) @parse_ini_file(KREXX_DIR . 'kreXX.ini', true);
		}
		
		// exists ?
		//
		return (isset($_config[$group][$name])) ? $_config[$group][$name] : self::$configFallback[$group][$name];
	}
	
	/**
	 * Simply outputs the Header of kreXX
	 *
	 * @access protected
	 * @static
	 */
	protected static function _outputHeader($headline) {
		static $docType = null;
		
		// Setting tamplate info
		if (is_null(self::$skin)) {
			self::$skin = self::_config('skin', 'selected');
		}
		if (!isset($docType)) {
			// send doctype only once!
			$docType = self::_config('header', 'doctype');
			return self::renderHeader($docType, $headline);
		} else {
			return self::renderHeader('', $headline);
		}
	}
	
	/**
	 * Simply renders the footer and output current settings
	 *
	 * @var string $caller
	 * 		Where was kreXX initially invoced from
	 */
	protected static function _outputFooter($caller) {
		$config = (array) @parse_ini_file(KREXX_DIR . 'kreXX.ini', true);
		$path = KREXX_DIR . 'kreXX.ini';
		// now we need to stitch together the content of the ini file
		// as well as it's path
		if (isset($config[0]) && $config[0] === false) {
			// ini is not accessible
			// tell the user, that we are using fallback sttings
			$parameter = array (
					self::$configFallback 
			);
			$anonFunction = function ($parameter) {
				$data = $parameter[0];
				return self::_vars($data);
			};
			$configOutput = self::renderExpandableChild('Config not accessible!', 'Using fallback settings!', $anonFunction, $parameter);
		} else {
			// ini is accessible
			// but we should tell the user when we use fallback settings
			// we are using the fallback as 'master'
			// we do not test if the values from the ini make sense 
			foreach (self::$configFallback as $mainkey => $data) {
				foreach ($data as $subkey => $setting) {
					if (!isset($config[$mainkey][$subkey])) {
						$config[$mainkey][$subkey] = $setting . ' ***FALLBACK***';
					}
				}
			}
			$parameter = array (
					$config 
			);
			$anonFunction = function ($parameter) {
				$data = $parameter[0];
				return self::_vars($data);
			};
			$configOutput = self::renderExpandableChild('Current configuration', $path, $anonFunction, $parameter);
		}
		return self::renderFooter($caller, $configOutput);
	}
	
	/**
	 * Print the skin (CSS)
	 *
	 * @return boolean
	 * @access protected
	 * @static
	 */
	protected Static Function _css() {
		static $_css = false;
		
		// already set ?
		//
		if ($_css) {
			return;
		}
		
		// get the css file
		//
		$_ = KREXX_DIR . 'skins/' . self::$skin . '/skin.css';
		$css = file_get_contents($_);
		
		// print ?
		//
		if ($_css = $css != '') {
			$js = join(file(KREXX_DIR . 'skins/' . self::$skin . "/kreXX.js"));
			$js .= join(file(KREXX_DIR . "jquery-1.11.0.js"));
			$js .= join(file(KREXX_DIR . "jquery-ui-1.10.4.dragable.min.js"));
			return self::renderCssJs($css, $js);
		}
	}
	
	/**
	 * Dump information about a variable
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 */
	protected Static Function _dump(&$data, $name = '...') {
		// object ?
		//
		if (is_object($data)) {
			return self::_object($data, $name);
		}
		
		// array ?
		//
		if (is_array($data)) {
			return self::_array($data, $name);
		}
		
		// resource ?
		//
		if (is_resource($data)) {
			return self::_resource($data, $name);
		}
		
		// scalar ?
		//
		if (is_string($data)) {
			return self::_string($data, $name);
		}
		
		if (is_float($data)) {
			return self::_float($data, $name);
		}
		
		if (is_integer($data)) {
			return self::_integer($data, $name);
		}
		
		if (is_bool($data)) {
			return self::_boolean($data, $name);
		}
		
		// null ?
		//
		if (is_null($data)) {
			return self::_null($name);
		}
	}
	
	/**
	 * Render a dump for an object
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 * @todo refactor this method, it's too big
	 */
	protected Static Function _object($data, $name) {
		static $level = 0;
		
		$output = '';
		$parameter = array (
				$data,
				$name 
		);
		
		$level++;
		
		if (self::_isInHive($data)) {
			
			// tell them, we've been here before
			// but also say who we are
			$output .= self::renderRecursion($name, get_class($data), self::_generateDomIdFromObject($data));
			
			// we will not render this one, but since we
			// return to wherever we came from, we need to decrese the level
			$level--;
			return $output;
		}
		self::_addToHive($data);
		// analyse it only, if we are below a certain level.
		if ($level <= (int) self::_config('deep', 'level')) {
			$anonFunction = function (&$parameter) {
				$data = $parameter[0];
				$name = $parameter[1];
				$output = '';
				
				// Dumping all Properties
				// But only if we have any
				if (count(get_object_vars($data))) {
					$parameter = array (
							$data 
					);
					$anonFunction = function (&$parameter) {
						$data = $parameter[0];
						// standard dump of the vars
						return self::_vars($data);
					};
					$output .= self::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Public properties');
				} else {
					// output "Nothing found"
					// self::renderSingleChild('No Properties found', 'No Properties found', '', false, 'Properties');
				}
				
				// Dumping all protected properties
				if (self::_config('deep', 'analyseProtected') == 'true') {
					$ref = new \ReflectionClass($data);
					$refProps = $ref->getProperties(\ReflectionProperty::IS_PROTECTED);
					if (count($refProps)) {
						// I need to preprocess them, since I do not want to render a reflection property
						$default = $ref->getDefaultProperties();
						$privateProps = array ();
						/* @var ReflectionProperty $refProperty */
						foreach ($refProps as $refProperty) {
							$refProperty->setAccessible(true);
							$value = $refProperty->getValue($data);
							$propName = $refProperty->name;
							if (is_null($value)) {
								// no Value is set?
								// we might want to look at the default value
								$value = $default[$propName];
							}
							$privateProps[$propName] = $value;
						}
						
						$parameter = array (
								$privateProps 
						);
						$anonFunction = function (&$parameter) {
							$data = $parameter[0];
							// standard dump of the vars
							return self::_vars($data);
						};
						$output .= self::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Protected properties');
					}
				}
				
				// Dumping all private properties
				if (self::_config('deep', 'analysePrivate') == 'true') {
					$ref = new \ReflectionClass($data);
					$refProps = $ref->getProperties(\ReflectionProperty::IS_PRIVATE);
					if (count($refProps)) {
						// I need to preprocess them, since I do not want to render a reflection property
						$default = $ref->getDefaultProperties();
						$privateProps = array ();
						foreach ($refProps as $refProerty) {
							$refProerty->setAccessible(true);
							$value = $refProerty->getValue($data);
							$propName = $refProerty->name;
							if (is_null($value)) {
								// no Value is set?
								// we might want to look at the default value
								$value = $default[$propName];
							}
							$privateProps[$propName] = $value;
						}
						
						$parameter = array (
								$privateProps 
						);
						$anonFunction = function (&$parameter) {
							$data = $parameter[0];
							// standard dump of the vars
							return self::_vars($data);
						};
						$output .= self::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Private properties');
					}
				}
				
				// Dumping all methods
				// But only if we have any
				$methods = get_class_methods($data);
				sort($methods);
				foreach ($methods as $k => $v) {
					$myMethods[] = $v;
				}
				// Dump them only, if there actually are methods in there
				if (count($myMethods)) {
					if (self::_config('deep', 'analyseMethods') == 'true') {
						$ref = new \ReflectionClass($data);
					} else {
						// will not be uses, dummy object
						$ref = new \stdClass();
					}
					$parameter = array (
							$myMethods,
							$ref 
					);
					$anonFunction = function (&$parameter) {
						$myMethods = $parameter[0];
						$ref = $parameter[1];
						return self::_methods($myMethods, $ref);
					};
					$output .= self::renderExpandableChild($name, 'class internals', $anonFunction, $parameter, 'Methods');
				} else {
					// output "Nothing found"
					// self::renderSingleChild('No Methods found', 'No Methods found', '', false, 'Methods');
				}
				
				// Dumping traversable data
				if (self::_config('deep', 'analyseTraversable') == 'true') {
					if (is_a($data, 'Traversable')) {
						$traversable = iterator_to_array($data);
						$parameter = $traversable;
						$anonFunction = function (&$parameter) {
							$data = $parameter;
							// this could be anything, we need to examine it first, which happens in the _dump()
							return self::_dump($data);
						};
						$output .= self::renderExpandableChild($name, 'Foreach', $anonFunction, $parameter, 'Traversable Info');
					} else {
						// do nothing
					}
				}
				
				// Dumping all configured debug functions
				$funcList = explode(',', self::_config('deep', 'debugMethods'));
				foreach ($funcList as $funcName) {
					if (method_exists($data, $funcName)) {
						// @todo use an errorhandler to catch possible fatals here
						//       also, for some reasons does the @ not work in a Magento,
						//       strangely, when I poll the magento-debug functions with
						//       the default values, I get NULL in return
						//       looks like we have to stick with the warning, until I find
						//       a method to suppress it
						try {
// 							$reflection = new \ReflectionClass($data);
// 							$refDebugFunc =  $reflection->getMethod($funcName);
// 							$refParams = $refDebugFunc->getParameters();
							$args = array();
							// for each passable argument
							// we give him what he would expect
// 							foreach ($refParams as $refParam) {
// 								if (!$refParam->isDefaultValueAvailable()) {
// 									// no default, we give him a NULL
// 									$args[] = NULL;
// 								}
// 							}
							$parameter = call_user_func_array(array($data, $funcName), $args);
							// $parameter = $args;
						} catch (Exception $e) {
							$parameter = 'Could not retrive data from: ' .$funcName;
						}
						// if (! is_null($parameter)) {
							$anonFunction = function (&$parameter) {
								// $data = $parameter[0];
								return self::_dump($parameter);
							};
							$output .= self::renderExpandableChild($name, 'debug method', $anonFunction, $parameter, $funcName . ' Info');
						// }
					}
				}
				return $output;
			};
		} else {
			// we will not go into the object pyramide any further
			$anonFunction = function () {
				return self::_string("Maxlevel for object analysis reached. I will not go any further.\n To increase this value, please edit the ini file.", 'Feedback');
			};
		}
		// output data from the class
		$output .= self::renderExpandableChild($name, 'class', $anonFunction, $parameter, get_class($data), self::_generateDomIdFromObject($data));
		// we've finished this one, and can decrease the levelsetting
		$level--;
		return $output;
	}
	
	/**
	 * Render a dump for the properties of an array or objeect
	 *
	 * @param
	 *        	mixed &$data
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _vars(&$data) {
		$parameter = array (
				$data 
		);
		$analysis = function (&$parameter) {
			$output = '';
			$data = $parameter[0];
			$_is_object = is_object($data);
			
			$_recursion_marker = self::_marker();
			
			// recursion detected
			if (self::_isInHive($data, true)) {
				return self::renderRecursion();
				;
			}
			
			// remember, that we've already been here
			self::_addToHive($data);
			
			// keys ?
			$keys = ($_is_object) ? array_keys(get_object_vars($data)) : array_keys($data);
			
			// itterate
			foreach ($keys as $k) {
				
				// skip marker
				if ($k === $_recursion_marker) {
					continue;
				}
				
				// get real value
				if ($_is_object) {
					$v = & $data->$k;
				} else {
					$v = & $data[$k];
				}
				
				$output .= self::_dump($v, $k);
			}
			return $output;
		};
		return self::renderExpandableChild('', '', $analysis, $parameter);
	}
	
	/**
	 * Return the marked used to stain arrays
	 * and objects in order to detect recursions
	 *
	 * @return string
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _marker() {
		static $_recursion_marker;
		if (! isset($_recursion_marker)) {
			$_recursion_marker = uniqid('kreXX');
		}
		
		return $_recursion_marker;
	}
	
	/**
	 * Adds a variable to the hive of arrays and objects which
	 * are tracked for whether they have recursive entries
	 *
	 * @param
	 *        	mixed &$bee either array or object, not a scallar vale
	 * @return array all the bees
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function &_addToHive(&$bee) {
		
		// static $_ = array ();
		
		// new bee ?
		//
		if (! is_null($bee) && (is_array($bee) || is_object($bee))) {
			
			// stain it
			//
			$_recursion_marker = self::_marker();
			if (is_object($bee)) {
				// we do something else for objects
				// setting a recussion er inside might trigger a magical function.
				// some Zend Framework objects throw an error, while Varien Objects
				// redirect the recussion value to an internal array
				$objectHash = spl_object_hash($bee);
				if (! isset(self::$recussionHive[1][$objectHash])) {
					self::$recussionHive[1][$objectHash] = 0;
				}
				self::$recussionHive[1][$objectHash]++;
			} else {
				// this should be an array
				if (! isset($bee[$_recursion_marker])) {
					$bee[$_recursion_marker] = 0;
				}
				$bee[$_recursion_marker]++;
				self::$recussionHive[0][] = & $bee;
			}
		}
		return $bee;
	}
	
	/**
	 * Render a dump for the methods of an object
	 *
	 * @param
	 *        	array &$data
	 *        	the methods
	 * @param reflection_class $ref
	 *        	a reflection of the original class
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _methods(&$data, $ref) {
		$parameter = array (
				$data,
				$ref 
		);
		$analysis = function (&$parameter) {
			$data = $parameter[0];
			$ref = $parameter[1];
			$result = '';
			// itterate
			//
			if (self::_config('deep', 'analyseMethods') == 'true') {
				// Deep analysis of the methods
				foreach ($data as $method) {
					$reflection = $ref->getMethod($method);
					$methodData = array ();
					// get the comment
					$methodData['comments'] = nl2br(htmlSpecialChars($reflection->getDocComment()));
					// get declaration data
					$access = '';
					if ($reflection->isAbstract()) {
						$access .= 'abstract ';
					}
					if ($reflection->isPublic()) {
						$access .= 'public ';
					}
					if ($reflection->isFinal()) {
						$access .= 'final ';
					}
					if ($reflection->isPrivate()) {
						$access .= 'private ';
					}
					if ($reflection->isProtected()) {
						$access .= 'protected ';
					}
					if ($reflection->isStatic()) {
						$access .= 'static ';
					}
					$methodData['access'] = $access;
					// get declaration place
					$declaringClass = $reflection->getDeclaringClass();
					$methodData['declared in'] = htmlSpecialChars($declaringClass->getFileName()) . '<br>';
					$methodData['declared in'] .= htmlSpecialChars($declaringClass->getName()) . ' ';
					$methodData['declared in'] .= 'in line ' . htmlSpecialChars($reflection->getStartLine());
					// get parameters
					$parameters = $reflection->getParameters();
					foreach ($parameters as $key => $parameters) {
						$methodData[$key] = htmlSpecialChars($parameters);
					}
					
					$result .= self::_method($methodData, $method);
				}
				return $result;
			} else {
				// We just dump them
				return self::_array($data, 'methods');
			}
		};
		return self::render($analysis, $parameter);
	}
	
	/**
	 * Render a dump for method infos
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _method($data, $name) {
		$parameter = array (
				$data 
		);
		$anonFunction = function ($parameter) {
			$data = $parameter[0];
			$output = '';
			foreach ($data as $key => $string) {
				if ($key !== 'comments' && $key !== 'declared in') {
					$output .= self::renderSingleChild($string, $key, $string, false, 'reflection');
				} else {
					// security, there could be anything inside the string
					$output .= self::renderSingleChild($string, $key, '. . .', true, 'reflection');
				}
			}
			return $output;
		};
		return self::renderExpandableChild($name, 'method', $anonFunction, $parameter);
	}
	
	/**
	 * Render a dump for a NULL value
	 *
	 * @param string $name        	
	 * @return string
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _null($name) {
		$data = 'NULL';
		return self::renderSingleChild($data, $name, $data, false, 'null');
	}
	
	/**
	 * Render a dump for an array
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _array(&$data, $name) {
		// Dumping all Properties
		// But only if we have any
		if (count($data)) {
			$parameter = array (
					$data 
			);
			$anonFunction = function ($parameter) {
				$data = $parameter[0];
				return self::_vars($data);
			};
		}
		return self::renderExpandableChild($name, 'array', $anonFunction, $parameter, count($data) . ' elements');
	}
	
	/**
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _resource($data, $name) {
		$data = get_resource_type($data);
		return self::renderSingleChild($data, $name, $data, false, 'resource');
	}
	
	/**
	 * Render a dump for a boolean value
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _boolean($data, $name) {
		$data = $data ? 'TRUE' : 'FALSE';
		return self::renderSingleChild($data, $name, $data, false, 'boolean');
	}
	
	/**
	 * Render a dump for a integer value
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _integer($data, $name) {
		return self::renderSingleChild($data, $name, $data, false, 'integer');
	}
	
	/**
	 * Render a dump for a float value
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _float($data, $name) {
		return self::renderSingleChild($data, $name, $data, false, 'float');
	}
	
	/**
	 * Render a dump for a string value
	 *
	 * @param mixed $data        	
	 * @param string $name        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function _string($data, $name) {
		
		// extra ?
		//
		$_extra = false;
		$_ = $data;
		if (strLen($data) > KREXX_TRUNCATE_LENGTH) {
			$_ = substr($data, 0, KREXX_TRUNCATE_LENGTH - 3) . '...';
			$_extra = true;
		}
		
		// security, there could be anything inside the string
		$data = nl2br(htmlSpecialChars($data));
		$_ = nl2br(htmlSpecialChars($_));
		
		return self::renderSingleChild($data, $name, $_, $_extra, 'string');
	}
	
	/**
	 * Dump information about a variable
	 *
	 * @param mixed $data,...        	
	 * @access protected
	 * @static
	 *
	 *
	 *
	 */
	protected Static Function dump($data, $headline = '') {
		// find caller
		//
		$_ = debug_backtrace();
		while ($caller = array_pop($_)) {
			if (strToLower(@$caller['function']) == 'krexx' || strToLower(@$caller['class']) == 'krexx') {
				break;
			}
		}
		if (is_object($data)) {
			$headline = get_class($data);
		}
		
		$maxCall = (int) self::_config('output', 'maxCall');
		if (self::$kreXXCount >= $maxCall) {
			// called too often, we might get into trouble here!
			return;
		}
		
		// Start Output
		self::$kreXXCount++;
		$output = self::_outputHeader($headline);
		$output .= self::_css();
		
		$output .= self::_dump($data);
		$output .= self::_outputFooter($caller);
		if (self::_config('output', 'destination') == 'file') {
			// save it to a file
			self::_saveOutputToFile($output);
		} else {
			// send it to the browser
			echo $output;
		}
		// we flush this one anyway
		// typo3 6.2 renders only as much characters as it actually has
		// inside the content cache. kreXX actually has no access there,
		// so anything additional will delete stuff from the page button.
		// But when we fulsh it, everything is fine.
		ob_end_flush();
		// }
		// cleanup
		self::_cleanupHive();
	}
	
	/**
	 * Resets all Arrays inside the recussion array
	 */
	protected static function _cleanupHive() {
		// remove all recussion marker inside of arrays
		$recursionMarker = self::_marker();
		if (isset(self::$recussionHive[0]) && count(self::$recussionHive[0])) {
			foreach (self::$recussionHive[0] as $i => $bee) {
				try {
					unset(self::$recussionHive[0][$i][$recursionMarker]);
				} catch (Exception $e) {
					// do nothing
				}
			}
		}
		// reset the array, to get it clean for the next run
		self::$recussionHive = array ();
	}
	
	/**
	 * Find out if our bee is already in the hive
	 * Do we have a recussion here?
	 *
	 * @param mixed $bee        	
	 * @return boolean
	 * @todo rework whole object analysis and remove the $arrayOnly option here
	 */
	protected static function _isInHive($bee, $arrayOnly = false) {
		// test for references in order to
		// prevent endless recursion loops
		//
		$recursionMarker = self::_marker();
		if (is_object($bee) && ! $arrayOnly) {
			// retrieve a possible hash
			$objectHash = spl_object_hash($bee);
			$recussionValue = @self::$recussionHive[1][$objectHash];
		} else {
			// retrieve a possible value
			$recussionValue = @$data[$recursionMarker];
		}
		
		$recussionValue = (int) $recussionValue;
		if ($recussionValue > 0) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Generates a id for the DOM
	 * This is used to jump from a Recussion to the object analysis data
	 * The ID is the object hash as well as the kruXX call number, to avoit
	 * collusions (even if they are unlikely).
	 *
	 * @param mixed $data        	
	 * @return string the generated id
	 */
	protected static function _generateDomIdFromObject($data) {
		if (is_object($data)) {
			return self::$kreXXCount . '_' . spl_object_hash($data);
		} else {
			// do nothing
		}
	}
	
	/**
	 * Logges the output of kreXX to a file and cleans up and
	 * cleans up the output folder
	 *
	 * @param string $output        	
	 */
	protected static function _saveOutputToFile($output) {
		static $logDir;
		if (is_null($logDir)) {
			$logDir = self::_config('logging', 'folder') . DIRECTORY_SEPARATOR;
		}
		
		$logList = glob(KREXX_DIR . $logDir . "*");
		array_multisort(array_map('filemtime', $logList), SORT_DESC, $logList);
		$maxFileCount = (int) self::_config('logging', 'maxfiles');
		$count = 1;
		foreach ($logList as $file) {
			if (is_file($file) && $count >= $maxFileCount) {
				unlink($file);
			}
			$count++;
		}
		
		static $timestamp = 0;
		
		if ($timestamp == 0) {
			$timestamp = time();
		}
		
		// now we need to write this into a file
		$filename = KREXX_DIR . $logDir . DIRECTORY_SEPARATOR . $timestamp . '.kreXX.html';
		// file_put_contents($filename, $buffer);
		file_put_contents($filename, $output, FILE_APPEND);
	}
}