<?php
/**
 * kreXX: Krumo eXXtended
 *
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was written by:
 * @author Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 * 
 * 
 * 
 * @license http://opensource.org/licenses/LGPL-2.1 GNU Lesser General Public License Version 2.1
 * @package kreXX
 */


namespace kreSpace;

/**
 * kreXX renderer
 *
 * This class hosts the internal rendering functions
 *
 * @package kreXX
 */
class kreXXRender {
	
	/**
	 * Counts how often kreXX was called
	 *
	 * @var integer
	 */
	protected static $kreXXCount = 0;
	
	/**
	 * Name of the skin currently in use
	 * Gets set as soon as the css is being loaded
	 * 
	 * @var string
	 * @see self::_outputHeader();
	 */
	protected static $skin;
	
	/**
	 * Return kreXX version
	 *
	 * @return string
	 * @access public
	 * @static
	 *
	 */
	Public Static Function version() {
		return '1.0.0 rc2';
	}
	
	/**
	 *
	 * @param string $data
	 *        The initial data rendered
	 * @param string $name
	 *        The Name, what we render here
	 * @param string $normal
	 *        The normal content. conetnt using linebreaks shoulg get rendert into $extra
	 * @param boolean $extra
	 *        If set, the data text will be rendert inside the yellow square
	 * @todo refactor this function into some smaler ones to make it better readable
	 */
	protected static function renderSingleChild($data, $name = '', $normal = '', $extra = false, $type = '') {
		if ($type == 'string') {
			$strlen = ' ' . strlen($data) . ' characters';
		} else {
			$strlen = '';
		}
		// this one is a little bit more complicated than the others,
		// because it assembels some partials and stitches them together
		$template = self::getTemplateFileContent('singleChild');
		$partExpand = '';
		$partToggle = '';
		$partCallable = '';
		$partExtra = '';
		if ($extra) {
			// we have a lot of text, so we render this one expandeble (yellow box)
			$partExpand = self::getTemplateFileContent('singleChildExpand');
			$partToggle = self::getTemplateFileContent('singleChildToggle');
		}
		if (is_callable($data)) {
			// Add callable partial
			$partCallable = self::getTemplateFileContent('singleChildCallable');
		}
		if ($extra) {
			// Add the yellow box for large output text
			$partExtra = self::getTemplateFileContent('singleChildExtra');
		}
		// stitching it together
		$template = str_replace('{expand}', $partExpand, $template);
		$template = str_replace('{toggle}', $partToggle, $template);
		$template = str_replace('{callable}', $partCallable, $template);
		$template = str_replace('{extra}', $partExtra, $template);
		$template = str_replace('{name}', $name, $template);
		$template = str_replace('{type}', $type, $template);
		$template = str_replace('{strlen}', $strlen, $template);
		$template = str_replace('{normal}', $normal, $template);
		$template = str_replace('{data}', $data, $template);
		return $template;
	}
	
	/**
	 *  Render a block of a detected recursion.
	 *  If the recussion is an objekct, a click should jump to the original analysis data
	 * 
	 * @param string $name
	 * 		we might want to tell the user how to actually access it
	 * @param string $value
	 * 		we might want to tell the user what this actually is
	 * @param string $domId
	 * 		the id of the analysis data, a click on the recussion should jump to it
	 */
	protected Static Function renderRecursion($name = '', $value = '', $domId = '') {
		$template = self::getTemplateFileContent('recursion');
		// Replace our stuff in the partial
		$template = str_replace('{name}', $name, $template);
		$template = str_replace('{domId}', $domId, $template);
		$template = str_replace('{value}', $value, $template);
		return $template;

	}
	
	
	/**
	 * Renders the kreXX header
	 * 
	 * @param string $doctype
	 */
	protected static function renderHeader($doctype, $headline) {
		$template = self::getTemplateFileContent('header');
		// Replace our stuff in the partial
		$template = str_replace('{version}', self::version(), $template);
		$template = str_replace('{doctype}', $doctype, $template);
		$template = str_replace('{kreXXCount}', self::$kreXXCount, $template);
		$template = str_replace('{headline}', $headline, $template);
		return $template;
	}
	
	/**
	 * Renders the kreXX footer
	 * 
	 * @see renderHeader()
	 */
	protected static function renderFooter($caller, $configOutput){
		$template = self::getTemplateFileContent('footer');
		// Replace our stuff in the partial
		if (!isset($caller['file'])) {
			$caller['file'] = '';
			$caller['line'] = '';
		}
		$template = str_replace('{callerFile}', $caller['file'], $template);
		$template = str_replace('{callerLine}', $caller['line'], $template);
		$template = str_replace('{configInfo}', $configOutput, $template);
		return $template;
	}
	
	/**
	 * Renders a nest with a anonymous function in the middle
	 * 
	 * @param anonymousfunction $anonFunction
	 * @param array $parameter
	 * @param string $domId
	 */
	protected static function renderNest($anonFunction, &$parameter, $domId = '') {
		$template = self::getTemplateFileContent('nest');
		// Replace our stuff in the partial
		$template = str_replace('{domId}', $domId, $template);
		return str_replace('{mainfunction}', call_user_func($anonFunction, $parameter), $template);
	}
	
	/**
	 * Pretty much the same as renderNest, bus without the Nest
	 * 
	 * @param anonymousfunction $anonFunction
	 * @param array $parameter
	 */
	protected static function render($anonFunction, &$parameter) {
		return call_user_func($anonFunction, $parameter);
	}

	/**
	 * Simply outputs ths css and js stuff
	 * 
	 * @param string $css
	 * @param string $js
	 */
	protected static function renderCssJs($css, $js) {
		$template = self::getTemplateFileContent('cssJs');
		// Replace our stuff in the partial
		$template = str_replace('{css}', $css, $template);
		$template = str_replace('{js}', $js, $template);
		return $template;
	}
	
	/**
	 * Renders a expandable child with a callback in the middle
	 * 
	 * @param anonymousfunction $anonFunction
	 * @param array $parameter
	 */
	protected static function renderExpandableChild($name, $type, $anonFunction, &$parameter, $additional = '', $domId = '') {
		if ($name == '' && $type == '') {
			// without a Name or Type I only display the Child with a Node.
			$template = self::getTemplateFileContent('expandableChildSimple');
			// Replace our stuff in the partial
			return str_replace('{mainfunction}', self::render($anonFunction, $parameter), $template);
		} else {
			// we need to render this one normally
			$template = self::getTemplateFileContent('expandableChildNormal');
			// Replace our stuff in the partial
			$template = str_replace('{name}', $name, $template);
			$template = str_replace('{type}', $type, $template);
			$template = str_replace('{additional}', $additional, $template);
			return str_replace('{nest}', self::renderNest($anonFunction, $parameter, $domId), $template);
		}
	}
	
	/**
	 * Loads a template file from the skin folder
	 * 
	 * @param string $what
	 * 		Filename in the skinfolder without the ".html" at the end
	 */
	protected static function getTemplateFileContent($what) {
		$file = KREXX_DIR . 'skins/' . self::$skin . '/' . $what . '.html';
		$content = file_get_contents($file);
		// remove whitespace
		return preg_replace('/\s+/', ' ',$content);
	}
}