/**
 * JavaScript routines for kreXX
 * 
 * kreXX is a debugging tool (PHP5.3 only), which displays structured information
 * about any PHP object. It is a nice replacement for print_r() or var_dump()
 * which are used by a lot of PHP developers.
 * @author brainworXX GmbH <info@brainworxx.de>
 *
 * kreXX is a fork of Krumo, which was written by:
 * @author Kaloyan K. Tsvetkov <kaloyan@kaloyan.info>
 * @license http://opensource.org/licenses/lgpl-license.php GNU Lesser General Public License Version 2.1
 *
 * @package kreXX
 */

/////////////////////////////////////////////////////////////////////////////

/**
* kreXX JS Class
*/
function kreXX() {
	}

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

/**
* Add a CSS class to an HTML element
*
* @param HtmlElement el 
* @param string className 
* @return void
*/
kreXX.reclass = function(el, className) {
	if (el.className.indexOf(className) < 0) {
		el.className += (' ' + className);
		}
	};

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

/**
* Remove a CSS class to an HTML element
*
* @param HtmlElement el 
* @param string className 
* @return void
*/
kreXX.unclass = function(el, className) {
	if (el.className.indexOf(className) > -1) {
		el.className = el.className.replace(className, '');
		}
	};

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

/**
* Toggle the nodes connected to an HTML element
*
* @param HtmlElement el 
* @return void
*/
kreXX.toggle = function(el) {
	var ul = el.parentNode.getElementsByTagName('ul');
	for (var i=0; i<ul.length; i++) {
		if (ul[i].parentNode.parentNode == el.parentNode) {
			ul[i].parentNode.style.display = (ul[i].parentNode.style.display == 'none')
				? 'block'
				: 'none';
			}
		}

	// toggle class
	//
	if (ul[0].parentNode.style.display == 'block') {
		kreXX.reclass(el, 'kreXX-opened');
		} else {
		kreXX.unclass(el, 'kreXX-opened');
		}
	};

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

/**
* Hover over an HTML element
*
* @param HtmlElement el 
* @return void
*/
kreXX.over = function(el) {
	kreXX.reclass(el, 'kreXX-hover');
	kreXX.unclass(el, 'kreXX-highlight');
	};

// -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 

/**
* Hover out an HTML element
*
* @param HtmlElement el 
* @return void
*/

kreXX.out = function(el) {
	kreXX.unclass(el, 'kreXX-hover');
	};
	
/**
 * Expand kreXX child and then focus on it 
 * 
 * @param string dimId
 * @return void
 */
kreXX.jumpTo = function(domId) {
	(function( $ ) {
		$(function() {
			nests = $('#'+domId).parents('.kreXX-nest');
			// show them
			nests.show();
			// expand parents
			nests.prev().addClass('kreXX-opened');
			// mark it and scroll into view
			expandableElement = $('#'+domId).prev();
			expandableElement.addClass('kreXX-highlight');
			$('html, body').animate({
	            scrollTop: expandableElement.offset().top
	        }, 5);
		});
	})(jQuery);
};
/////////////////////////////////////////////////////////////////////////////

/**
* Select the whole element on click
* 
* @param HtmlElement el 
* @return void
*/
//function select_all(el) {
//    if (typeof window.getSelection != "undefined" && typeof document.createRange != "undefined") {
//        var range = document.createRange();
//        range.selectNodeContents(el);
//        var sel = window.getSelection();
//        sel.removeAllRanges();
//        sel.addRange(range);
//    } else if (typeof document.selection != "undefined" && typeof document.body.createTextRange != "undefined") {
//        var textRange = document.body.createTextRange();
//        textRange.moveToElementText(el);
//        textRange.select();
//    }
//}